%%
clear all;
close all;
tic
%parameters of the code
ncol=100;
nline=1000;
nmod=100;
x=linspace(0,2*pi,ncol);
Amax=2;

%paramters of the synthetic data
% A_synth=[1.8];
A_synth=[0.8 0.8 1.4];
% phi_synth=[pi/6];
phi_synth=[3*pi/4 -pi/4 pi/6];
% thick_synth=[5];
thick_synth=[5 5 9];
% depth_synth=[0];
depth_synth=[3 0 0];

%displays the values defining the synthetic data
n_synth=length(depth_synth);
str=[ 'A0 =' , num2str(A_synth(1,:)) ];
disp(str);
str=[ 'phi0 =' , num2str(phi_synth(1,:)) ];
disp(str);
str=[ 'depth0 =' , num2str(depth_synth(1,:)) ];
disp(str);

%creates a matrix with the synthetic data in it
mat=zeros(nline,ncol);
for k=1:n_synth
    f=A_synth(1,k)*cos(x-phi_synth(1,k))+depth_synth(1,k);
    for i=1:ncol
        line=nline/2+1+round(nmod*f(1,i));
        mat(nline/2+1+round(nmod*f(1,i))-thick_synth(k):nline/2+1+round(nmod*f(1,i))+thick_synth(k),i)=thick_synth(k);
    end
end

% max_thick=max(thick_synth);
% rand_th=0.98;
% rand_mat=rand(nline,ncol);
% x_rand=find(rand_mat>rand_th);
% n=length(x_rand);
% for i=1:n
%     mat(x_rand(i))=rand*max_thick;
% end


% %designs an FMI type image with gaps
% j=1;
% mat_void=zeros(nline,ncol);
% for i=1:ncol
%     if j>0
%         mat_void(:,i)=1;
%     end
%     if j==15
%         j=-10;
%     end
%     j=j+1;
% end
% mat=mat.*mat_void;


%plots the sinusoids 
figure
pcolor(mat), shading interp
colormap(hot)
set(gca,'XTickLabel',2*pi/10:2*pi/10:2*pi)
xlabel('Phi (rad)')
ylabel('Depth (mm)')


%% Finds events with a certain thickness in every trace
min_thick=3;
max_thick=30;
delta_res_thresh=3;
mat_edge=zeros(nline,ncol);

for j=1:ncol
    i=1;
    while i<=nline-max_thick-1
        if abs(mat(i,j)-mat(i+1,j))>=delta_res_thresh
            ii=1;
            while ii<max_thick
                if abs(mat(i+ii,j)-mat(i+ii+1,j))>=delta_res_thresh
                    break
                else
                    ii=ii+1;
                end
            end
            if ii>=min_thick && ii<max_thick
                mat_edge(i+floor(ii/2),j)=ii;
            end
            i=i+ii+1;
        else
            i=i+1;
        end
    end
end

mat_edge=round(mat_edge);
pcolor(mat_edge), shading interp;
colormap(hot)
set(gca,'XTickLabel',2*pi/10:2*pi/10:2*pi)
xlabel('Phi (rad)')
ylabel('Depth (mm)')

% mat_edge=edge(mat);
% image(mat_edge)

%% Correlation edges Regroups the points together in groups thanks to the correlations
n_free=21; %odd number freedom of search
n_free_2=(n_free-1)/2;
mat_corre=zeros(nline,ncol,4);
corre_thresh=0.7;

%initialisation j=1
mat_edge(1:max_thick,:)=0;
mat_edge(nline-max_thick+1:nline,:)=0;
i_edge=find(mat_edge(:,1)>0);
n=length(i_edge);
k=0;
for i=1:n
    if mod(mat_edge(i_edge(i),1),2)==0
        vec=mat(i_edge(i)-mat_edge(i_edge(i),1)/2:i_edge(i)+mat_edge(i_edge(i),1)/2-1,1);
        vec_mat=mat(i_edge(i)-mat_edge(i_edge(i),1)/2-n_free_2:i_edge(i)+mat_edge(i_edge(i),1)/2-1+n_free_2,2);
    else
        vec=mat(i_edge(i)-(mat_edge(i_edge(i),1)-1)/2:i_edge(i)+(mat_edge(i_edge(i),1)-1)/2,1);
        vec_mat=mat(i_edge(i)-(mat_edge(i_edge(i),1)-1)/2-n_free_2:i_edge(i)+(mat_edge(i_edge(i),1)-1)/2+n_free_2,2);
    end
    [corre,i_corre]=max_corre(vec,vec_mat,n_free);
    i_corre=i_corre-n_free_2-1;
    
    if corre>=corre_thresh
        mat_corre(i_edge(i),1,1)=corre;
        mat_corre(i_edge(i)+i_corre,2,1)=corre;
        mat_corre(i_edge(i),1,2)=mat_edge(i_edge(i),1);
        mat_corre(i_edge(i)+i_corre,2,2)=mat_edge(i_edge(i),1);
        k=k+1;
        mat_corre(i_edge(i),1,3)=k;
        mat_corre(i_edge(i)+i_corre,2,3)=k;
        mat_corre(i_edge(i)+i_corre,2,4)=i_corre;
    end
end

i_corre_thresh=3;
%for all the other j
for j=2:ncol-1
    mat_edge(1:max_thick,:)=0;
    mat_edge(nline-max_thick+1:nline,:)=0;
    i_edge=find(mat_edge(:,j));
    n=length(i_edge);
    
    %Makes the regrouping if necessary
    for i=1:n
        if mod(mat_edge(i_edge(i),j),2)==0
            vec=mat_corre(i_edge(i)-mat_edge(i_edge(i),j)/2:i_edge(i)+mat_edge(i_edge(i),j)/2-1,j,1);
        else
            vec=mat_corre(i_edge(i)-(mat_edge(i_edge(i),j)-1)/2:i_edge(i)+(mat_edge(i_edge(i),j)-1)/2,j,1);
        end
        i_edge_find=find(vec);
        if isempty(i_edge_find)~=1
            m=length(i_edge_find);
            if m==1
                temp_i=i_edge(i);
                if mod(mat_edge(i_edge(i),j),2)==0
                    i_edge(i)=i_edge(i)-mat_edge(i_edge(i),j)/2+i_edge_find-1;
                else
                    i_edge(i)=i_edge(i)-(mat_edge(i_edge(i),j)-1)/2+i_edge_find-1;
                end
                mat_edge(temp_i,j)=0;
                mat_edge(i_edge(i),j)=mat_corre(i_edge(i),j,2);
            end
        end
    end
    
    %Goes on with the points in a column
    for i=1:n
        if mod(mat_edge(i_edge(i),j),2)==0
            vec=mat(i_edge(i)-mat_edge(i_edge(i),j)/2:i_edge(i)+mat_edge(i_edge(i),j)/2-1,j);
            vec_mat=mat(i_edge(i)-mat_edge(i_edge(i),j)/2-n_free_2:i_edge(i)+mat_edge(i_edge(i),j)/2-1+n_free_2,j+1);
        else
            vec=mat(i_edge(i)-(mat_edge(i_edge(i),j)-1)/2:i_edge(i)+(mat_edge(i_edge(i),j)-1)/2,j);
            vec_mat=mat(i_edge(i)-(mat_edge(i_edge(i),j)-1)/2-n_free_2:i_edge(i)+(mat_edge(i_edge(i),j)-1)/2+n_free_2,j+1);
        end
        [corre,i_corre]=max_corre(vec,vec_mat,n_free);
        i_corre=i_corre-n_free_2-1;

        if corre>=corre_thresh
            if abs(i_corre-mat_corre(i_edge(i),j,4))<i_corre_thresh
                if mat_corre(i_edge(i),j,3)==0
                    k=k+1;
                    mat_corre(i_edge(i),j,3)=k;
                    mat_corre(i_edge(i)+i_corre,j+1,3)=k;
                else
                    mat_corre(i_edge(i)+i_corre,j+1,3)=mat_corre(i_edge(i),j,3);
                end
            end
            if mat_corre(i_edge(i),j,4)==0 || mat_corre(i_edge(i),j,4)~=0&&abs(i_corre-mat_corre(i_edge(i),j,4))<i_corre_thresh
                mat_corre(i_edge(i),j,1)=corre;
                mat_corre(i_edge(i)+i_corre,j+1,1)=corre;
                mat_corre(i_edge(i),j,2)=mat_edge(i_edge(i),j);
                mat_corre(i_edge(i)+i_corre,j+1,2)=mat_edge(i_edge(i),j);
                mat_corre(i_edge(i)+i_corre,j+1,4)=i_corre;
            end
        end
    end
end

%%
figure
pcolor(mat_corre(:,:,3)), shading interp
colormap(jet)
set(gca,'XTickLabel',2*pi/10:2*pi/10:2*pi)
xlabel('Phi (rad)')
ylabel('Depth (mm)')

%% Processing of the regrouped points
%finds the arguments of the best fitting sinusoid
n_points_sin=5;
mat_fit=zeros(ncol,6,k);
mysin=@(Amplitude, Phase, Depth ,x) Amplitude * cos( x-Phase ) + Depth;

kk=0;
for i=1:k
    [ifind,jfind] = find(mat_corre(:,:,3)==i);
    if length( ifind ) >= n_points_sin
        m=mean(ifind);
        sinfit=fit( x(jfind)' , ifind , mysin , 'StartPoint', [100,0,m]);
        kk=kk+1;
        mat_fit(1,1,kk)=length( ifind );
        mat_fit(1:length(ifind) ,2,kk)=ifind;
        mat_fit(1:length(ifind) ,3,kk)=jfind;
        mat_fit(1,4,kk)=sinfit.Amplitude;
        mat_fit(1,5,kk)=sinfit.Phase;
        mat_fit(1,6,kk)=sinfit.Depth;
    end
end
mat_fit(:,:,kk+1:end)=[];

%regroups the points belonging to the same sinusoid together and
%recalculates the arguments of the sinusoid
prec=0.1;
for i=1:kk
    j=i+1;
    while j<kk+1
        if abs((mat_fit(1,4,j)-mat_fit(1,4,i))/mat_fit(1,4,i))<prec...
                && abs((mat_fit(1,5,j)-mat_fit(1,5,i))/mat_fit(1,5,i))<prec...
                && abs((mat_fit(1,6,j)-mat_fit(1,6,i))/mat_fit(1,6,i))<prec
            ifind=vertcat(mat_fit(1:mat_fit(1,1,i),2,i),mat_fit(1:mat_fit(1,1,j),2,j));
            jfind=vertcat(mat_fit(1:mat_fit(1,1,i),3,i),mat_fit(1:mat_fit(1,1,j),3,j));
            mat_fit(1,1,i)=mat_fit(1,1,i)+mat_fit(1,1,j);

            sinfit=fit( x(jfind)' , ifind , mysin , 'StartPoint', [mat_fit(1,4,i),mat_fit(1,5,i),mat_fit(1,6,i)]);
            mat_fit(1:length(ifind) ,2,i)=ifind;
            mat_fit(1:length(ifind) ,3,i)=jfind;
            mat_fit(1,4,i)=sinfit.Amplitude;
            mat_fit(1,5,i)=sinfit.Phase;
            mat_fit(1,6,i)=sinfit.Depth;

            mat_fit(:,:,j)=[];
            j=j-1;
            kk=kk-1;
        end
        j=j+1;
    end
end

%removes all the groups containing less than n_points_sin_tot points
n_points_sin_tot=8;
i=1;
while i<kk+1
    if mat_fit(1,1,i)<n_points_sin_tot
        mat_fit(:,:,i)=[];
        i=i-1;
        kk=kk-1;
    end
    i=i+1;
end

mat_fit(1,4,:)=mat_fit(1,4,:)/100;
mat_fit(1,6,:)=(mat_fit(1,6,:)-500)/100;
mat_fit(1,:,:)

mat_final=zeros(nline,ncol);
for i=1:kk
    for j=1:mat_fit(1,1,i)
        mat_final(mat_fit(j,2,i),mat_fit(j,3,i))=i;
    end
end

figure
pcolor(mat_final), shading interp
toc
% figure
% pcolor(mat_edge), shading interp
