%%
clear all;
close all;

tic

%parameters of the code
n_max=5;
ncol=100;
nline=1000;
nmod=1000;
Amax=0.2;
size_win=2*Amax*nmod;
x=linspace(0,2*pi,ncol);
y=linspace(-Amax,Amax,2*Amax*nmod+1);
ncos=100;
ndef=100;
delta_h=0.01;
delta_step=10;%on suppose une mesure tous les mms
n_win=floor(((nline/nmod)-2*Amax)/delta_h)+1;
depth=zeros(n_win,1);
depth(1,1)=-0.3;
for i=2:n_win
    depth(i,1)=depth(i-1,1)+delta_h;
end

%paramters of the synthetic data
% A_synth=[0.8];
A_synth=[0.08 0.08 0.14];
% phi_synth=[pi/6];
phi_synth=[3*pi/4 -pi/4 pi/6];
% depth_synth=[0];
depth_synth=[0.3 0 0];

%displays the values defining the synthetic data
n_synth=length(depth_synth);
str=[ 'A0 =' , num2str(A_synth(1,:)) ];
disp(str);
str=[ 'phi0 =' , num2str(phi_synth(1,:)) ];
disp(str);
str=[ 'Depth0 =' , num2str(depth_synth(1,:)) ];
disp(str);

%creates a matrix with the synthetic data in it
mat=zeros(nline,ncol);
for k=1:n_synth
    f=A_synth(1,k)*cos(x-phi_synth(1,k))+depth_synth(1,k);
    for i=1:ncol
        mat(nline/2+1+round(nmod*f(1,i)),i)=1;
    end
end


rand_th=0.98;
rand_mat=rand(nline,ncol);
mat(rand_mat>rand_th)=1;

%designs an FMI type image with gaps
j=1;
mat_void=zeros(nline,ncol);
for i=1:ncol
    if j>0
        mat_void(:,i)=1;
    end
    if j==15
        j=-10;
    end
    j=j+1;
end
mat=mat.*mat_void;

%plots the sinusoids 
figure
pcolor(mat), shading interp
colormap(hot)
set(gca,'XTickLabel',2*pi/10:2*pi/10:2*pi)
xlabel('Phi (rad)')
ylabel('Depth (mm)')
tic
%% Draw the lines in the parameter space
l=1;
mat_para=zeros(ndef+1,ndef+1,n_win);
for ii=1:n_win
    mat_win=mat(l:l+size_win-1,:);
    l=l+delta_step;
    mat_para(:,:,ii)=sin_hough(mat_win,x,y,ndef,ncos,Amax);
end
toc
%% Plots the results in the parameter space
Apas=Amax/ndef;
A=0:Apas:Amax;
phipas=2*pi/ndef;
phi=-pi:phipas:pi;

figure
pcolor(phi,A(1:end-5),mat_para(1:end-5,:,61)), shading interp
xlabel('Phase')
ylabel('Amplitude')
colorbar

%% Find local maxima corresponding to the parameters of the sinusoids
[Maxima,MaxPos,Minima,MinPos]=MinimaMaxima3D(mat_para(1:end-5,:,:),1,1,n_max,0);

%% Reorders the results and keeps only the clearest sinusoids
m_min=10;
n_sinus=0;
for i=1:n_max
    if mat_para(MaxPos(i,1),MaxPos(i,2),MaxPos(i,3))>=m_min
        n_sinus=n_sinus+1;
    end
end
win_sinus=zeros(n_sinus,3);
j=1;
for i=1:n_max
    if mat_para(MaxPos(i,1),MaxPos(i,2),MaxPos(i,3))>=m_min
        win_sinus(j,1)=A(MaxPos(i,1));
        win_sinus(j,2)=phi(MaxPos(i,2));
        win_sinus(j,3)=depth(MaxPos(i,3));
        j=j+1;
    end
end

%% Test the sinusoids found previously
%the sinusoids have to be close enough to a certain amount of points and
%the standard deviation has also to be small enough
delta_h=4;
n_tresh=30;
std_thresh=1000;
mat_std=mat;

for i=1:n_sinus
    f=win_sinus(i,1)*cos(x-win_sinus(i,2))+win_sinus(i,3);
    k=0;
    sum_k=0;
    for j=1:ncol
        l=nline/2+1+round(nmod*f(1,j));
        [temp_mat,~]=find(mat(l-delta_h:l+delta_h,j)==1);
        temp_mat=temp_mat-l;
        [mm,m]=size(temp_mat);
        
        if mm>0&&m>0
            k=k+m;
            for jj=1:m
                sum_k=sum_k+temp_mat(1,jj)^2;
            end
        end
    end
    if k>1
        std=sqrt(sum_k/(k-1));
    end

    if k>=n_tresh&&std<=std_thresh
        str=[ 'True A =' , num2str(win_sinus(i,1)) ];
        disp(str);
        str=[ 'True phi =' , num2str(win_sinus(i,2)) ];
        disp(str);
        str=[ 'True depth =' , num2str(win_sinus(i,3)) ];
        disp(str);
        
        
        f=win_sinus(i,1)*cos(x-win_sinus(i,2))+win_sinus(i,3);
        for ii=1:ncol
            mat_std(nline/2+1+round(nmod*f(1,ii)),ii)=mat_std(nline/2+1+round(nmod*f(1,ii)),ii)+1;
        end
    end
end


%% Display

figure
pcolor(mat_std), shading interp
colormap(hot)
set(gca,'XTickLabel',2*pi/10:2*pi/10:2*pi)
xlabel('Phi (rad)')
ylabel('Depth (mm)')

toc