%%
clear all;
close all;
tic
%parameters of the code
ncol=100;
nline=1000;
nmod=100;
x=linspace(0,2*pi,ncol);

%paramters of the synthetic data
% A_synth=[0.8];
A_synth=[0.8 0.8 1.4];
% phi_synth=[pi/6];
phi_synth=[3*pi/4 -pi/4 pi/6];
% depth_synth=[0];
depth_synth=[3 0 0];

%displays the values defining the synthetic data
n_synth=length(depth_synth);
str=[ 'phi0 =' , num2str(phi_synth(1,:)) ];
disp(str);
str=[ 'A0 =' , num2str(A_synth(1,:)) ];
disp(str);
str=[ 'Depth0 =' , num2str(depth_synth(1,:)) ];
disp(str);

%creates a matrix with the synthetic data in it
mat=zeros(nline,ncol);
for k=1:n_synth
    f=A_synth(1,k)*cos(x-phi_synth(1,k))+depth_synth(1,k);
    for i=1:ncol
        mat(nline/2+1+round(nmod*f(1,i)),i)=1;
    end
end


% rand_th=0.999;
% rand_mat=rand(nline,ncol);
% mat(rand_mat>rand_th)=1;
% 
% %designs an FMI type image with gaps
% j=1;
% mat_void=zeros(nline,ncol+1);
% for i=1:ncol+1
%     if j>0
%         mat_void(:,i)=1;
%     end
%     if j==15
%         j=-10;
%     end
%     j=j+1;
% end
% mat=mat.*mat_void;


%plots the sinusoids 
figure
pcolor(mat), shading interp
colormap(hot)
set(gca,'XTickLabel',2*pi/10:2*pi/10:2*pi)
xlabel('Phi (rad)')
ylabel('Depth (mm)')

%% Creates the orientation space
n_max=1;
thr_rad=0;
[Y0,X0]=find(mat==1);
n_edge=length(X0);
mat_or=zeros(nline,ncol);
cst_mat_or=5;
mat_or(:,:,2)=cst_mat_or*ones(nline,ncol);
n_x_line=1000;
n_factor=3;%odd number. Used to refine the matrix.
n_factor_2=(n_factor-1)/2;
size_win=10;
n_thresh=10;

phi_min_thresh=20*pi/180;
n_phi_rad=41; %odd number
phi_rad_1=linspace(0,phi_min_thresh,n_phi_rad);
phi_rad_2=linspace(pi-phi_min_thresh,pi,n_phi_rad);
phi_rad=horzcat(phi_rad_1,phi_rad_2);
n_phi_rad=2*n_phi_rad-1;

% % Creates a smaller window in which to perform the radon transform
% n_phi_cir=9001;
% phi_cir=linspace(0,pi,n_phi_cir);
% win_cir=create_cir_win( phi_cir , size_win , n_factor);


% Finds the path in the matrix along which the sum is performed
[cell_path_rad]=find_path( phi_rad , size_win , n_x_line , n_factor );
n_cell_path_rad=length(cell_path_rad);

% Creates a bigger matrix to remove the effect of edges
mat_enlarged=zeros(nline+2*size_win,ncol+2*size_win);
mat_enlarged(size_win+1:nline+size_win,1:size_win)=mat(:,ncol-size_win+1:ncol);
mat_enlarged(size_win+1:nline+size_win,size_win+1:ncol+size_win)=mat;
mat_enlarged(size_win+1:nline+size_win,ncol+size_win+1:end)=mat(:,1:size_win);

% Creates a larger matrix large_mat with a size of n_factor*length(mat_enlarged) to
% increase the resolution
if n_factor==1
    XXL_mat=mat;
else
    XXL_mat=zeros((nline+2*size_win)*n_factor,(ncol+2*size_win)*n_factor);
    for i=1:nline+2*size_win
        for j=1:ncol+2*size_win
            if mat_enlarged(i,j)~=0
                for ii=1:n_factor
                    for jj=1:n_factor
                        XXL_mat((i-1)*n_factor+ii,(j-1)*n_factor+jj)=mat_enlarged(i,j);
                    end
                end
            end
        end
    end
end

% Computes the sum of lines with different angles in a matrix centred on an edge point
% Core of the radon transform
figure
for i=1:n_edge
    mat_local=XXL_mat((Y0(i)-1)*n_factor+1:(Y0(i)+2*size_win)*n_factor,...
        (X0(i)-1)*n_factor+1:(X0(i)+2*size_win)*n_factor);
    
    sum_cell_path_rad=zeros(n_cell_path_rad+2,1);
    for j=1:n_cell_path_rad
        sum_cell_path_rad(j+1,1)=sum(mat_local(cell_path_rad{1,j}));
    end
    sum_cell_path_rad(1,1)=sum_cell_path_rad(n_cell_path_rad+1,1);
    sum_cell_path_rad(n_cell_path_rad+2,1)=sum_cell_path_rad(2,1);
    
    [pks,locs] = findpeaks(sum_cell_path_rad , 'NPEAKS', n_max, 'SORTSTR','descend');
    if isempty(pks)~=1
        if pks(1)>=n_thresh
            mat_or(Y0(i),X0(i),1)=pks(1);
            plot3(Y0(i),X0(i),mat_or(Y0(i),X0(i),1),'b+'), hold on
            mat_or(Y0(i),X0(i),2)=cell_path_rad{2,locs(1)}(3,1);
        end
    end
end
hold off

figure
pcolor(mat_or(:,:,2)), shading interp
colormap(hot)
set(gca,'XTickLabel',2*pi/10:2*pi/10:2*pi)
xlabel('Phi (rad)')
ylabel('Depth (mm)')

%% Reconstruct the sinusoid thanks to the radon method 
%In this method NO blank column is required
%Looks for the points where the derivative is zero and tries to regroup all
%these points two by two. From the position of these max and min of the sin
%function it is easy to get the arguments of the sin function
phi=0;
n_i_del=10;
phi_del=pi/6;
n_j_del=round(ncol*phi_del/(2*pi));
ind_points_2=zeros(1000,2);

mat_or_2=cst_mat_or*ones(nline+2*n_i_del,ncol+2*n_j_del);
mat_or_2(n_i_del+1:nline+n_i_del,n_j_del+1:ncol+n_j_del)=mat_or(:,:,2);
n_ind_thresh=5;

i=0;
while phi<=phi_min_thresh
    phi=min(min(mat_or_2));
    if phi<cst_mat_or
        [i_phi,j_phi]=find(mat_or_2==phi);
        if sum(sum((mat_or_2(i_phi(1)-n_i_del:i_phi(1)+n_i_del,j_phi(1)-n_j_del:j_phi(1)+n_j_del)-cst_mat_or)~=0))>=n_ind_thresh
            i=i+1;
            ind_points_2(i,1:2)=[i_phi(1)-n_i_del,j_phi(1)-n_j_del];
        end
        mat_or_2(i_phi(1)-n_i_del:i_phi(1)+n_i_del,j_phi(1)-n_j_del:j_phi(1)+n_j_del)=cst_mat_or*ones(2*n_i_del+1,2*n_j_del+1);
        if j_phi(1)<n_j_del+1
            mat_or_2(i_phi(1)-n_i_del:i_phi(1)+n_i_del,ncol+n_j_del+(j_phi(1)-n_j_del):ncol+n_j_del)=cst_mat_or*ones(2*n_i_del+1,abs(j_phi(1)-n_j_del-1));
        end
        if j_phi(1)>ncol-n_j_del
            mat_or_2(i_phi(1)-n_i_del:i_phi(1)+n_i_del,n_j_del+1:n_j_del+(n_j_del-(ncol-j_phi(1))))=cst_mat_or*ones(2*n_i_del+1,n_j_del-(ncol-j_phi(1)));
        end
    end
end
n_ind_points_2=i;

figure
pcolor(mat_or(:,:,2)), shading interp

%identifies if the sinusoid is oritend towards the top or the
%bottom at that point

%regroups the points two by two by checking the period, the amplitude and
%by testing the solution on the edge map
mat_edge=mat;
delta_h=15;
n_tresh=40;
ind_points=zeros(n_ind_points_2,4);
temp_ind=zeros(n_ind_points_2-1,2);
n_col_T=ncol/2;
Amax=2;
prec_T=10;
n_line_A=2*Amax*nmod+1;
n_loop=2;
p=0;
for k=1:n_loop
    i=1;
    while i<n_ind_points_2+1
        l=0;
        for j=i+1:n_ind_points_2
            if abs(ind_points_2(i,1)-ind_points_2(j,1))<=n_line_A && abs(abs(ind_points_2(i,2)-ind_points_2(j,2))-ncol/2)<=prec_T
                [Amplitude , Phase , Depth] = find_arg(ind_points_2(i,:),ind_points_2(j,:),ncol);
                if test_sinusoid(mat_edge , Amplitude , Phase , Depth , delta_h , x )>=n_tresh
                    l=l+1;
                    temp_ind(l,:)=ind_points_2(j,:);
                    temp=j;
                end
            end
        end
        if l==1
            p=p+1;
            ind_points(p,1:2)=ind_points_2(i,:);
            ind_points(p,3:4)=temp_ind(1,:);
            ind_points_2(temp,:)=[];
            n_ind_points_2=n_ind_points_2-1;
        end
        i=i+1;
    end
end
ind_points(p+1:end,:)=[];

arg_sinus=zeros(p,3);
for i=1:p
    [arg_sinus(i,1),arg_sinus(i,2),arg_sinus(i,3)]=find_arg(ind_points(i,1:2),ind_points(i,3:4),ncol);
end
arg_sinus
%%
for i=1:p
    f=arg_sinus(i,1)*cos(x-arg_sinus(i,2))+arg_sinus(i,3);
    for ii=1:ncol
        mat(round(f(1,ii)),ii)=mat(round(f(1,ii)),ii)+1;
    end
end

figure
pcolor(mat), shading interp
colormap(hot)
set(gca,'XTickLabel',2*pi/10:2*pi/10:2*pi)
xlabel('Phi (rad)')
ylabel('Depth (mm)')
toc